@@ -1,14 +1,22 @@
 # Import all special functions before registering the Numba module
 # type inferer
+
+from ._version import get_versions
+__version__ = get_versions()['version']
+del get_versions
+
 from numba.special import *
-from numba import module_type_inference
 
 import os
 import sys
 import logging
 from numba import typesystem
 
-__version__ = '0.5.0'
+
+
+def get_include():
+    numba_root = os.path.dirname(os.path.abspath(__file__))
+    return os.path.join(numba_root, "include")
 
 # NOTE: Be sure to keep the logging level commented out before commiting.  See:
 #   https://github.com/numba/numba/issues/31
@@ -91,9 +99,9 @@ try:
 except ImportError:
     signal_to_name = {}
 else:
-    signal_to_name = { signal_code:signal_name
+    signal_to_name = dict((signal_code, signal_name)
                            for signal_name, signal_code in vars(signal).items()
-                               if signal_name.startswith("SIG") }
+                               if signal_name.startswith("SIG"))
 
 
 def test(whitelist=None, blacklist=None):
@@ -138,13 +146,22 @@ def test(whitelist=None, blacklist=None):
 def nose_run(module=None):
     import nose.config
     import __main__
-    config = nose.config.Config()
-    config.configure(['--logging-level=DEBUG',
-                      '--verbosity=3',      # why is this ignored?
-                      # '--with-doctest=1', # not recognized?
-                      #'--doctest-tests'
-                      ])
-    config.verbosity = 3
-    nose.run(module=module or __main__, config=config)
+
+    #os.environ["NOSE_EXCLUDE"] = "(test_all|test_all_noskip|.*compile_with_pycc.*|bytecode)"
+    #os.environ["NOSE_VERBOSE"] = "4"
+
+    result = nose.main()
+    return len(result.errors), len(result.failures)
 
 __all__ = typesystem.__all__ + decorators.__all__ + special.__all__
+
+from numba.typesystem import map_dtype
+from numba.type_inference.module_type_inference import (is_registered,
+                                                        register,
+                                                        register_inferer,
+                                                        get_inferer,
+                                                        register_unbound,
+                                                        register_callable)
+
+from numba.typesystem.typeset import *
+__all__.extend(["numeric", "floating", "complextypes"])
