@@ -6,19 +6,18 @@ import llvm.core as lc
 import llvm.passes as lp
 import llvm.ee as le
 
-from .llvm_types import _int1, _int32, _intp, _LLVMCaster
+from .llvm_types import _int1, _int32, _LLVMCaster
 from .multiarray_api import MultiarrayAPI # not used
 from .symtab import Variable
 from numba import typesystem
-from numba.typesystem import BuiltinType
 
 from numba import *
 from . import visitors, nodes, llvm_types, utils, function_util
 from .minivect import minitypes, llvm_codegen
-from numba import ndarray_helpers, translate, error, extension_types
-from numba.typesystem import is_obj, promote_closest, promote_to_native
+from numba import ndarray_helpers, error, extension_types
+from numba.typesystem import is_obj, promote_to_native
 from numba.utils import dump
-from numba import naming
+from numba import naming, metadata
 from numba.functions import keep_alive
 
 import logging
@@ -360,7 +359,7 @@ class RefcountingMixin(object):
         "Py_XDECREF a temporary"
         assert not self.nopython
         decref = decref or self.decref
-        decref(self.builder.load(temp))
+        decref(self.load_tbaa(temp, object_))
 
     def xincref_temp(self, temp):
         "Py_XINCREF a temporary"
@@ -522,11 +521,15 @@ class LLVMCodeGenerator(visitors.NumbaVisitor, ComplexSupportMixin,
                 variable = self.symtab[argname]
                 variable.lvalue = self._allocate_arg_local(argname, argtype,
                                                            larg)
+
             #else:
             #    raise error.InternalError(argname, argtype)
 
             self.incref_arg(argname, argtype, larg, variable)
 
+            if variable.type.is_array:
+                self.preload_attributes(variable, variable.lvalue)
+
     def c_array_to_pointer(self, name, stackspace, var):
         "Decay a C array to a pointer to allow pointer access"
         ltype = var.type.base_type.pointer().to_llvm(self.context)
@@ -597,6 +600,7 @@ class LLVMCodeGenerator(visitors.NumbaVisitor, ComplexSupportMixin,
         self.caster = _LLVMCaster(self.builder)
         self.object_coercer = ObjectCoercer(self)
         self.multiarray_api.set_PyArray_API(self.llvm_module)
+        self.tbaa = metadata.TBAAMetadata(self.llvm_module)
 
         self.object_local_temps = {}
         self._init_constants()
@@ -654,7 +658,10 @@ class LLVMCodeGenerator(visitors.NumbaVisitor, ComplexSupportMixin,
             # Done code generation
             del self.builder  # release the builder to make GC happy
 
-            logger.debug("ast translated function: %s" % self.lfunc)
+            if logger.level >= logging.DEBUG:
+                # logger.debug("ast translated function: %s" % self.lfunc)
+                logger.debug(self.llvm_module)
+
             # Verify code generation
             self.llvm_module.verify()  # only Module level verification checks everything.
 
@@ -707,6 +714,11 @@ class LLVMCodeGenerator(visitors.NumbaVisitor, ComplexSupportMixin,
                     phi.add_incoming(incoming_var.lvalue,
                                      parent_block.exit_block)
 
+                    if phi_node.type.is_array:
+                        nodes.update_preloaded_phi(phi_node.variable,
+                                                   incoming_var,
+                                                   parent_block.exit_block)
+
 
     def get_ctypes_func(self, llvm=True):
         import ctypes
@@ -834,7 +846,7 @@ class LLVMCodeGenerator(visitors.NumbaVisitor, ComplexSupportMixin,
 
     def visit_FunctionWrapperNode(self, node):
         global debug_conversion
-        from numba import ast_type_inference, pipeline
+        from numba import  pipeline
 
         was_debug_conversion = debug_conversion
         if debug_conversion:
@@ -869,16 +881,7 @@ class LLVMCodeGenerator(visitors.NumbaVisitor, ComplexSupportMixin,
 
         if not is_obj(node.signature.return_type):
             # Check for error using PyErr_Occurred()
-            # TODO: make this an option in CheckErrorNode
-            check_err = nodes.CheckErrorNode(
-                    nodes.ptrtoint(function_util.external_call(
-                                              self.context,
-                                              self.llvm_module,
-                                             'PyErr_Occurred')),
-                    goodval=nodes.ptrtoint(nodes.NULL))
-            func_call = func_call.cloneable
-            func_call = nodes.ExpressionNode(stmts=[func_call, check_err],
-                                             expr=func_call.clone)
+            func_call = nodes.PyErr_OccurredNode(func_call)
 
         # Coerce and return result
         if node.signature.return_type.is_void:
@@ -912,11 +915,30 @@ class LLVMCodeGenerator(visitors.NumbaVisitor, ComplexSupportMixin,
             bb = self.builder.basic_block
         lhs = self.llvm_alloca(type or llvm_types._pyobject_head_struct_p,
                                name=name, change_bb=False)
-        self.generate_assign_stack(self.visit(nodes.NULL_obj), lhs)
+        self.generate_assign_stack(self.visit(nodes.NULL_obj), lhs,
+                                   tbaa_type=object_)
         if change_bb:
             self.builder.position_at_end(bb)
         return lhs
 
+    def load_tbaa(self, ptr, tbaa_type, name=''):
+        """
+        Load a pointer and annotate with Type Based Alias Analysis
+        metadata.
+        """
+        instr = self.builder.load(ptr, name='')
+        self.tbaa.set_tbaa(instr, tbaa_type)
+        return instr
+
+    def store_tbaa(self, value, ptr, tbaa_type):
+        """
+        Load a pointer and annotate with Type Based Alias Analysis
+        metadata.
+        """
+        instr = self.builder.store(value, ptr)
+        if metadata.is_tbaa_type(tbaa_type):
+            self.tbaa.set_tbaa(instr, tbaa_type)
+
     def puts(self, msg):
         const = nodes.ConstNode(msg, c_string_type)
         self.visit(function_util.external_call(self.context,
@@ -999,6 +1021,7 @@ class LLVMCodeGenerator(visitors.NumbaVisitor, ComplexSupportMixin,
         result = self.visit(node.value)
         value_is_reference = node.value.type.is_reference
 
+        # TODO: TBAA for loads
         if isinstance(node.ctx, ast.Load):
             if value_is_reference:
                 # Struct reference, load result
@@ -1019,7 +1042,9 @@ class LLVMCodeGenerator(visitors.NumbaVisitor, ComplexSupportMixin,
     def visit_StructVariable(self, node):
         return self.visit(node.node)
 
-    def generate_assign_stack(self, lvalue, ltarget, decref=False, incref=False):
+    def generate_assign_stack(self, lvalue, ltarget,
+                              tbaa_node=None, tbaa_type=None,
+                              decref=False, incref=False):
         """
         Generate assignment operation and automatically cast value to
         match the target type.
@@ -1033,7 +1058,14 @@ class LLVMCodeGenerator(visitors.NumbaVisitor, ComplexSupportMixin,
             # Py_XDECREF any previous object
             self.xdecref_temp(ltarget)
 
-        self.builder.store(lvalue, ltarget)
+        instr = self.builder.store(lvalue, ltarget)
+
+        # Set TBAA on store instruction
+        if tbaa_node is None:
+            assert tbaa_type is not None
+            tbaa_node = self.tbaa.get_metadata(tbaa_type)
+
+        self.tbaa.set_metadata(instr, tbaa_node)
 
     def visit_Assign(self, node):
         target_node = node.targets[0]
@@ -1060,15 +1092,59 @@ class LLVMCodeGenerator(visitors.NumbaVisitor, ComplexSupportMixin,
                 decref = bool(self.loop_beginnings)
             else:
                 target = self.object_local_temps[target_node.id]
+
+            tbaa_node = self.tbaa.get_metadata(target_node.type)
         else:
             target = self.visit(target_node)
 
+            if isinstance(target_node, nodes.TempStoreNode):
+                # Use TBAA specific to only this temporary
+                tbaa_node = target_node.temp.get_tbaa_node(self.tbaa)
+            else:
+                # ast.Attribute | ast.Subscript  store.
+                # These types don't have pointer types but rather
+                # scalar values
+                if is_obj(target_node.type):
+                    target_type = object_
+                else:
+                    target_type = target_node.type
+                tbaa_type = target_type.pointer()
+                tbaa_node = self.tbaa.get_metadata(tbaa_type)
+
         # INCREF RHS. Note that new references are always in temporaries, and
         # hence we can only borrow references, and have to make it an owned
         # reference
-        self.generate_assign_stack(value, target,
+        self.generate_assign_stack(value, target, tbaa_node,
                                    decref=decref, incref=incref)
 
+        self.preload_attributes(target_node.variable, value)
+
+    def preload_attributes(self, var, value):
+        """
+        Pre-load ndarray attributes data/shape/strides.
+        """
+        if not (var.preload_data or var.preload_shape or var.preload_strides):
+            return
+
+        if not var.renameable:
+            # Stack allocated variable
+            var = self.builder.load(value)
+
+        acc = self.pyarray_accessor(value, var.type.dtype)
+
+        if var.preload_data:
+            var.preloaded_data = acc.data
+
+        if var.preload_shape:
+            shape = nodes.get_strides(self.builder, self.tbaa,
+                                      acc.shape, var.type.ndim)
+            var.preloaded_shape = tuple(shape)
+
+        if var.preload_strides:
+            strides = nodes.get_strides(self.builder, self.tbaa,
+                                        acc.strides, var.type.ndim)
+            var.preloaded_strides = tuple(strides)
+
     def visit_Num(self, node):
         if node.type.is_int:
             return self.generate_constant_int(node.n)
@@ -1101,13 +1177,14 @@ class LLVMCodeGenerator(visitors.NumbaVisitor, ComplexSupportMixin,
                        var.type.is_struct) and not var.is_constant
         if should_load and isinstance(node.ctx, ast.Load):
             # Not a renamed but an alloca'd variable
-            return self.builder.load(var.lvalue)
+            return self.load_tbaa(var.lvalue, var.type)
         else:
             return var.lvalue
 
-    def _handle_ctx(self, node, lptr, name=''):
+    def _handle_ctx(self, node, lptr, tbaa_type, name=''):
         if isinstance(node.ctx, ast.Load):
-            return self.builder.load(lptr, name=name and 'load_' + name)
+            return self.load_tbaa(lptr, tbaa_type,
+                                  name=name and 'load_' + name)
         else:
             return lptr
 
@@ -1277,6 +1354,9 @@ class LLVMCodeGenerator(visitors.NumbaVisitor, ComplexSupportMixin,
             phi = self.builder.phi(ltype, phi_node.variable.unmangled_name)
             phi_node.variable.lvalue = phi
 
+            if phi_node.variable.type.is_array:
+                nodes.make_preload_phi(self.context, self.builder, phi_node)
+
     def setblock(self, cfg_basic_block):
         if cfg_basic_block.is_fabricated:
             return
@@ -1655,7 +1735,7 @@ class LLVMCodeGenerator(visitors.NumbaVisitor, ComplexSupportMixin,
 
     def visit_DereferenceNode(self, node):
         result = self.visit(node.pointer)
-        return self.builder.load(result)
+        return self.load_tbaa(result, node.type.pointer())
 
     def visit_PointerFromObject(self, node):
         return self.visit(node.node)
@@ -1677,7 +1757,7 @@ class LLVMCodeGenerator(visitors.NumbaVisitor, ComplexSupportMixin,
 
         lptr = self.builder.gep(value, indices)
         if node.slice.type.is_int:
-            lptr = self._handle_ctx(node, lptr)
+            lptr = self._handle_ctx(node, lptr, node.value.type)
 
         return lptr
 
@@ -1685,8 +1765,8 @@ class LLVMCodeGenerator(visitors.NumbaVisitor, ComplexSupportMixin,
         assert node.node.type.is_array
         lvalue = self.visit(node.node)
         lindices = self.visit(node.slice)
-        lptr = node.subscript(self, lvalue, lindices)
-        return self._handle_ctx(node, lptr)
+        lptr = node.subscript(self, self.tbaa, lvalue, lindices)
+        return self._handle_ctx(node, lptr, node.type.pointer())
 
     #def visit_Index(self, node):
     #    return self.visit(node.value)
@@ -1772,6 +1852,7 @@ class LLVMCodeGenerator(visitors.NumbaVisitor, ComplexSupportMixin,
 
         if node.signature.struct_by_reference:
             if minitypes.pass_by_ref(node.signature.return_type):
+                # TODO: TBAA
                 result = self.builder.load(return_value)
 
         return result
@@ -1857,7 +1938,11 @@ class LLVMCodeGenerator(visitors.NumbaVisitor, ComplexSupportMixin,
         return node.llvm_temp
 
     def visit_TempLoadNode(self, node):
-        return self.builder.load(self.visit(node.temp))
+        # TODO: use unique type for each temporary load and store pair
+        temp = self.visit(node.temp)
+        instr = self.builder.load(temp, invariant=node.invariant)
+        self.tbaa.set_metadata(instr, node.temp.get_tbaa_node(self.tbaa))
+        return instr
 
     def visit_TempStoreNode(self, node):
         return self.visit(node.temp)
@@ -1876,7 +1961,8 @@ class LLVMCodeGenerator(visitors.NumbaVisitor, ComplexSupportMixin,
         # Assign value
         self.builder.position_at_end(bb)
         rhs = self.visit(node.node)
-        self.generate_assign_stack(rhs, lhs, decref=self.in_loop)
+        self.generate_assign_stack(rhs, lhs, tbaa_type=object_,
+                                   decref=self.in_loop)
 
         # goto error if NULL
         # self.puts("checking error... %s" % error.format_pos(node))
@@ -1884,11 +1970,11 @@ class LLVMCodeGenerator(visitors.NumbaVisitor, ComplexSupportMixin,
         # self.puts("all good at %s" % error.format_pos(node))
 
         if node.incref:
-            self.incref(self.builder.load(lhs))
+            self.incref(self.load_tbaa(lhs, object_))
 
         # Generate Py_XDECREF(temp) at end-of-function cleanup path
         self.xdecref_temp_cleanup(lhs)
-        result = self.builder.load(lhs, name=name + '_load')
+        result = self.load_tbaa(lhs, object_, name=name + '_load')
 
         if not node.type == object_:
             dst_type = node.type.to_llvm(self.context)
@@ -1922,9 +2008,14 @@ class LLVMCodeGenerator(visitors.NumbaVisitor, ComplexSupportMixin,
 
         return llvm_value
 
+    def pyarray_accessor(self, llvm_array_ptr, dtype):
+        acc = ndarray_helpers.PyArrayAccessor(self.builder, llvm_array_ptr,
+                                              self.tbaa, dtype)
+        return acc
+
     def visit_ArrayAttributeNode(self, node):
         array = self.visit(node.array)
-        acc = ndarray_helpers.PyArrayAccessor(self.builder, array)
+        acc = self.pyarray_accessor(array, node.array_type.dtype)
 
         attr_name = node.attr_name
         if attr_name == 'shape':
