@@ -1,19 +1,20 @@
 __all__ = ['autojit', 'jit', 'export', 'exportmany']
 
+import types
 import functools
 import logging
-import types
 import inspect
 
 from numba import *
-from numba import typesystem
-from . import utils, functions, ast_translate as translate, ast_type_inference
+from numba import typesystem, numbawrapper
+from . import utils, functions, ast_translate as translate
 from numba import translate as bytecode_translate
-from numba import error, pipeline, extension_type_inference
+from numba import  pipeline, extension_type_inference
 from .minivect import minitypes
 from numba.utils import debugout
 from numba.intrinsic import default_intrinsic_library
 from numba.external import default_external_library
+from numba.external.utility import default_utility_library
 from numba import double, int_
 import llvm.core as _lc
 
@@ -23,6 +24,7 @@ context.numba_pipeline = pipeline.Pipeline
 function_cache = context.function_cache = functions.FunctionCache(context)
 context.intrinsic_library = default_intrinsic_library(context)
 context.external_library = default_external_library(context)
+context.utility_library = default_utility_library(context)
 pipeline_env = pipeline.PipelineEnvironment.init_env(
     context, "Top-level compilation environment (in %s).\n" % __name__)
 
@@ -142,8 +144,6 @@ logger = logging.getLogger(__name__)
 #        import numpy
 #        return numpy.vectorize(func)
 
-from numpy import vectorize
-
 # The __tr_map__ global maps from Python functions to a Translate
 # object.  This added reference prevents the translator and its
 # generated LLVM code from being garbage collected when we leave the
@@ -153,56 +153,6 @@ from numpy import vectorize
 
 __tr_map__ = {}
 
-class NumbaFunction(object):
-    """
-    Numba function.
-
-        py_func: original Python function
-        ctypes_func: LLVM function wrapper, callable from Python
-        signature: minitype FunctionType signature
-        lfunc: LLVM function
-        methoddef: PyMethodDef ctypes structure for the wrapper function
-    """
-
-    def __init__(self, py_func, wrapper=None, ctypes_func=None, signature=None,
-                 lfunc=None):
-        self.py_func = py_func
-        self.wrapper = wrapper
-        self.ctypes_func = ctypes_func
-        self.signature = signature
-        self.lfunc = lfunc
-
-        name = py_func.__name__
-        if signature is not None:
-            name = signature.name or name
-        self.func_name = self.__name__ = name
-        self.func_doc = self.__doc__ = py_func.__doc__
-        self.__module__ = py_func.__module__
-
-    def __repr__(self):
-        if self.ctypes_func:
-            compiled = 'compiled numba function (%s)' % self.signature
-        else:
-            compiled = 'specializing numba function'
-
-        return '<%s %s>' % (compiled, self.py_func)
-
-    def __call__(self, *args, **kwargs):
-        if self.ctypes_func:
-            return self.invoke_compiled(self.ctypes_func, *args, **kwargs)
-        else:
-            if kwargs:
-                raise error.NumbaError("Cannot handle keyword arguments yet")
-
-            nargs = self.py_func.func_code.co_argcount
-            if len(args) != nargs:
-                raise error.NumbaError("Expected %d arguments, got %d" % (
-                                                        nargs, len(args)))
-            return self.wrapper(self, *args, **kwargs)
-
-    def invoke_compiled(self, compiled_numba_func, *args, **kwargs):
-        return compiled_numba_func(*args, **kwargs)
-
 def jit_extension_class(py_class, translator_kwargs):
     llvm_module = translator_kwargs.get('llvm_module')
     if llvm_module is None:
@@ -249,20 +199,23 @@ def _autojit2(template_signature, target, nopython, **translator_kwargs):
         types. Uses the AST translator backend. For the bytecode translator,
         use @autojit.
         """
-        @functools.wraps(f)
-        def wrapper(numba_func, *args, **kwargs):
+        def compile_function(args, kwargs):
+            "Compile the function given its positional and keyword arguments"
             signature = resolve_argtypes(numba_func, template_signature,
                                          args, kwargs, translator_kwargs)
             dec = jit2(restype=signature.return_type,
-                        argtypes=signature.args,
-                        target=target, nopython=nopython,
-                        **translator_kwargs)
-            compiled_numba_func = dec(f)
-            return numba_func.invoke_compiled(compiled_numba_func, *args, **kwargs)
+                       argtypes=signature.args,
+                       target=target, nopython=nopython,
+                       **translator_kwargs)
+
+            compiled_function = dec(f)
+            return compiled_function
 
-        f.live_objects = []
-        numba_func = numba_function_autojit_targets[target](f, wrapper=wrapper)
         function_cache.register(f)
+        cache = function_cache.get_autojit_cache(f)
+
+        wrapper = autojit_wrappers[(target, 'ast')]
+        numba_func = wrapper(f, compile_function, cache)
         return numba_func
 
     return _autojit2_decorator
@@ -277,7 +230,7 @@ def _autojit(target, nopython):
         @autojit2
         """
         @functools.wraps(f)
-        def wrapper(numba_func, *args, **kwargs):
+        def wrapper(args, kwargs):
             # Infer argument types
             arguments = args + tuple(kwargs[k] for k in sorted(kwargs))
             types = tuple(context.typemapper.from_python(value)
@@ -294,10 +247,12 @@ def _autojit(target, nopython):
                 compiled_numba_func = decorator(f)
                 _func_cache[types] = compiled_numba_func
 
-            return numba_func.invoke_compiled(compiled_numba_func, *args, **kwargs)
+            return compiled_numba_func
 
-        f.live_objects = []
-        numba_func = numba_function_autojit_targets[target](f, wrapper=wrapper)
+        pyfunc_cache = function_cache.register(f)
+
+        wrapper = autojit_wrappers[(target, 'bytecode')]
+        numba_func = wrapper(f, wrapper, pyfunc_cache)
         return numba_func
 
     return _autojit_decorator
@@ -333,10 +288,6 @@ def _jit2(restype=None, argtypes=None, nopython=False,
             argtys = []
 
         assert argtys is not None
-
-        if not hasattr(func, 'live_objects'):
-            func.live_objects = []
-
         function_cache.register(func)
 
         assert kwargs.get('llvm_module') is None # TODO link to user module
@@ -347,8 +298,8 @@ def _jit2(restype=None, argtypes=None, nopython=False,
                                                  ctypes=False,
                                                  **kwargs)
         signature, lfunc, wrapper_func = result
-        return NumbaFunction(func, ctypes_func=wrapper_func,
-                             signature=signature, lfunc=lfunc)
+        return numbawrapper.NumbaCompiledWrapper(func, wrapper_func,
+                                                 signature, lfunc)
 
     return _jit2_decorator
 
@@ -392,11 +343,12 @@ def _jit(restype=None, argtypes=None, backend='bytecode', **kws):
 
 jit_targets = {
     ('cpu', 'bytecode') : _jit,
-    ('cpu', 'ast') : _jit2
+    ('cpu', 'ast') : _jit2,
 }
 
-numba_function_autojit_targets = {
-    'cpu': NumbaFunction,
+autojit_wrappers = {
+    ('cpu', 'bytecode') : numbawrapper.NumbaSpecializingWrapper,
+    ('cpu', 'ast')      : numbawrapper.NumbaSpecializingWrapper,
 }
 
 def _process_sig(sigstr, name=None):
@@ -404,10 +356,13 @@ def _process_sig(sigstr, name=None):
     parts = sigstr.split()
     types_dict = dict(globals(), d=double, i=int_)    
     loc = {}
-    if len(parts) < 2:
+    # FIXME:  Need something more robust to differentiate between
+    #   name ret(arg1,arg2)
+    #   and ret(arg1, arg2) or ret ( arg1, arg2 )
+    if len(parts) < 2 or '(' in parts[0] or '[' in parts[0] or '('==parts[1][0]:
         signature = eval(sigstr, loc, types_dict)
         signature.name = None
-    else:
+    else: # Signature has a name
         signature = eval(' '.join(parts[1:]), loc, types_dict)
         signature.name = parts[0]
     if name is not None:
@@ -440,7 +395,7 @@ def jit(restype=None, argtypes=None, backend='ast', target='cpu', nopython=False
     deprecated as of the 0.3 release.*
     """
     kws.update(nopython=nopython, backend=backend)
-    if isinstance(restype, type):
+    if isinstance(restype, (type, types.ClassType)):
         cls = restype
         return jit_extension_class(cls, kws)
 
