@@ -38,6 +38,19 @@ def get_signature(ext_type, is_class, is_static, sig):
     restype = sig.return_type
     return minitypes.FunctionType(return_type=restype, args=argtypes)
 
+def get_classmethod_func(func):
+    """
+    Get the Python function the classmethod or staticmethod is wrapping.
+
+    In Python2.6 classmethod and staticmethod don't have the '__func__'
+    attribute.
+    """
+    if isinstance(func, classmethod):
+        return func.__get__(object()).im_func
+    else:
+        assert isinstance(func, staticmethod)
+        return func.__get__(object())
+
 def _process_signature(ext_type, method, default_signature,
                        is_static=False, is_class=False):
     """
@@ -71,8 +84,8 @@ def _process_signature(ext_type, method, default_signature,
             return None, None, None
 
         method, restype, argtypes = _process_signature(
-                        ext_type, method.__func__, default_signature,
-                        is_static=is_static, is_class=is_class)
+                    ext_type, get_classmethod_func(method),
+                    default_signature, is_static=is_static, is_class=is_class)
 
     return method, restype, argtypes
 
@@ -332,10 +345,10 @@ def create_extension(context, py_class, translator_kwargs):
 
     vtab, vtab_type = build_vtab(ext_type.vtab_type, method_pointers)
 
-    logger.info("struct: %s" % ext_type.attribute_struct)
-    logger.info("ctypes struct: %s" % ext_type.attribute_struct.to_ctypes())
+    logger.debug("struct: %s" % ext_type.attribute_struct)
+    logger.debug("ctypes struct: %s" % ext_type.attribute_struct.to_ctypes())
     extension_type = extension_types.create_new_extension_type(
             py_class.__name__, py_class.__bases__, class_dict,
             ext_type, vtab, vtab_type,
             lmethods, method_pointers)
-    return extension_type
\ No newline at end of file
+    return extension_type
