@@ -16,7 +16,7 @@ def _import_all():
 _import_all()
 
 def default_external_library(context):
-    '''Build an external functino library
+    '''Build an external function library
         
     context --- numba context
     '''
@@ -24,4 +24,4 @@ def default_external_library(context):
     
     for fncls in all.itervalues():
         extlib.add(fncls())
-    return extlib
\ No newline at end of file
+    return extlib
