@@ -6,7 +6,6 @@ define the transformations and the order in which they run on the AST.
 import inspect
 import ast as ast_module
 import logging
-import functools
 import pprint
 import random
 from timeit import default_timer as _timer
@@ -15,12 +14,11 @@ import llvm.core as lc
 
 import numba.closure
 from numba import error
-from numba import functions, naming, transforms, control_flow
-from numba import ast_type_inference as type_inference
+from numba import functions, naming, transforms, control_flow, optimize
 from numba import ast_constant_folding as constant_folding
 from numba import ast_translate
 from numba import utils
-from numba.utils import dump
+from numba.type_inference import infer as type_inference
 from numba.asdl import schema
 from numba.minivect import minitypes
 import numba.visitors
@@ -53,6 +51,8 @@ class Pipeline(object):
         'closure_type_inference',
         'transform_for',
         'specialize',
+        'optimize',
+        'preloader',
         'late_specializer',
         'fix_ast_locations',
         'cleanup_symtab',
@@ -176,11 +176,11 @@ class Pipeline(object):
 
             if __debug__ and logger.getEffectiveLevel() < logging.DEBUG:
                 te = _timer() #  for profiling individual stage
-                logger.info("%X pipeline stage %30s:\t%.3fms",
+                logger.debug("%X pipeline stage %30s:\t%.3fms",
                             id(self), method_name, (te - ts) * 1000)
 
         tomega = _timer() # for profiling complete pipeline
-        logger.info("%X pipeline entire:\t\t\t\t\t%.3fms",
+        logger.debug("%X pipeline entire:\t\t\t\t\t%.3fms",
                     id(self), (tomega - talpha) * 1000)
 
         return self.func_signature, self.symtab, ast
@@ -266,6 +266,12 @@ class Pipeline(object):
     def specialize(self, ast):
         return ast
 
+    def optimize(self, ast):
+        return ast
+
+    def preloader(self, ast):
+        return self.make_specializer(optimize.Preloader, ast).visit(ast)
+
     def late_specializer(self, ast):
         specializer = self.make_specializer(transforms.LateSpecializer, ast)
         return specializer.visit(ast)
