@@ -376,6 +376,6 @@ if __name__ == '__main__':
 #    signature_dec()()
 #    test_closure_outer_locals()
 #    test_closure_loop()
-    test_closure_outer_locals()
-#    import numba
-#    numba.testmod()
\ No newline at end of file
+#    test_closure_outer_locals()
+    import numba
+    numba.testmod()
