@@ -8,5 +8,4 @@ exportmany(['multf f4(f4,f4)', 'multi i4(i4,i4)'])(mult)
 export('multc c16(c16,c16)')(mult)
 export('mult f8(f8, f8)')(mult)
 
-print decorators.default_module
-
+print decorators.pipeline_env.exports.function_module_map
