@@ -24,7 +24,7 @@ class MyExtension(object):
     >>> obj.getvalue.__doc__
     'Return value'
     >>> type(obj.getvalue.im_func)
-    <type 'cython_function_or_method'>
+    <type 'numba_function_or_method'>
     >>> obj._numba_attrs._fields_
     [('value', <class 'ctypes.c_double'>)]
     """
@@ -120,4 +120,4 @@ class ExtensionTypeAsAttribute(object):
 if __name__ == '__main__':
 #    print ExtensionTypeAsAttribute.exttype
     import doctest
-    doctest.testmod()
\ No newline at end of file
+    doctest.testmod()
