@@ -4,7 +4,8 @@ import pprint
 
 from .minivect.complex_support import Complex64, Complex128, Complex256
 from .minivect import miniast, minitypes
-from . import _numba_types
+from numba import typesystem
+from numba.typesystem.typemapper import NumbaTypeMapper
 
 def itercode(code):
     """Return a generator of byte-offset, opcode, and argument
@@ -52,10 +53,13 @@ class NumbaContext(miniast.LLVMContext):
 
     def init(self):
         self.astbuilder = self.astbuilder_cls(self)
-        self.typemapper = _numba_types.NumbaTypeMapper(self)
+        self.typemapper = NumbaTypeMapper(self)
 
     def is_object(self, type):
-        return super(NumbaContext, self).is_object() or type.is_array
+        return super(NumbaContext, self).is_object(type) or type.is_array
+
+    def promote_types(self, *args, **kwargs):
+        return self.typemapper.promote_types(*args, **kwargs)
 
 def get_minivect_context():
     return NumbaContext()
@@ -80,8 +84,8 @@ def ast2tree (node, include_attrs = True):
         raise TypeError('expected AST, got %r' % node.__class__.__name__)
     return _transform(node)
 
-def pformat_ast (node, include_attrs = True):
-    return pprint.pformat(ast2tree(node, include_attrs))
+def pformat_ast (node, include_attrs = True, **kws):
+    return pprint.pformat(ast2tree(node, include_attrs), **kws)
 
 def dump(node):
-    print pformat_ast(node)
\ No newline at end of file
+    print pformat_ast(node)
